(function (drupalSettings) {
  let libraryPath = drupalSettings.audio_wavesurfer.library_path;

  const wavesurferPath = libraryPath + '/wavesurfer.esm.js';
  const wavesurferHoverPath = libraryPath + '/plugins/hover.esm.js';

  window.wavesurferLibrary = window.wavesurferLibrary || {};

  Promise.all([
    import(wavesurferPath),
    import(wavesurferHoverPath)
  ]).then(([WaveSurfer, Hover]) => {
/*    window.wavesurferLibrary.Wavesurfer = WaveSurfer.default;
    window.wavesurferLibrary.Hover = Hover.default;*/

    /**
     * Audio wavesurfer class.
     */
    class AudioWavesurfer {
      static currentPlayer = null;
      static currentPlayButtonIcon = null;
      constructor(containerElement, waveformPeaks, options) {
        this.containerElement = containerElement;
        this.waveformPeaks = waveformPeaks;
        this.options = options;
        this.colorOptions = this.options && this.options.hasOwnProperty('color_options') ? this.options.color_options : null;
        this.barOptions = this.options && this.options.hasOwnProperty('bar_options') ? this.options.bar_options : null;
        this.idAudio = containerElement.dataset.idAudio;
        this.pathAudio = containerElement.dataset.pathAudio;
        this.playButton = containerElement.querySelector(".play-button");
        this.playButtonIcon = containerElement.querySelector(".play-button-icon");
        this.currentTime = containerElement.querySelector(".time");
        this.totalDuration = containerElement.querySelector(".duration");
        this.wavesurfer = this.initWavesurfer();

        this.playButton.style.backgroundColor = this.colorOptions && this.colorOptions.hasOwnProperty('progress_color') ? this.colorOptions.progress_color : "#ffa84d";

        this.wavesurfer.registerPlugin(Hover.default.create({
          lineColor: this.colorOptions && this.colorOptions.hasOwnProperty('cursor_color') ? this.colorOptions.cursor_color : '#f28c28',
          lineWidth: 2,
          labelBackground: '#555',
          labelColor: '#fff',
          labelSize: '14px',
        }));

        this.setupEventListeners();
      }

      initWavesurfer() {
        const wavesurferParameters = {
          container: '#id-' + this.idAudio,
          height: 70,
          waveColor: this.colorOptions && this.colorOptions.hasOwnProperty('waveform_color') ? this.colorOptions.waveform_color : "#bdbebd",
          progressColor: this.colorOptions && this.colorOptions.hasOwnProperty('progress_color') ? this.colorOptions.progress_color : "#ffa84d",
          barWidth: this.barOptions && this.barOptions.hasOwnProperty('bar_width') ? parseInt(this.barOptions.bar_width) : 0,
          barGap: this.barOptions && this.barOptions.hasOwnProperty('bar_gap') ? parseInt(this.barOptions.bar_gap) : 0,
          barRadius: this.barOptions && this.barOptions.hasOwnProperty('bar_gap') ? parseInt(this.barOptions.bar_radius) : 0,
        };
        if (this.waveformPeaks) {
          wavesurferParameters.peaks = this.waveformPeaks;
          wavesurferParameters.media = new Audio(this.pathAudio);
        }

        return WaveSurfer.default.create(wavesurferParameters);
      }

      togglePlay() {
        if(this.wavesurfer.isPlaying()) {
          this.wavesurfer.pause();
          this.playButtonIcon.classList.remove('icon-pause2');
          this.playButtonIcon.classList.add('icon-play3');
        }
        else {
          if (AudioWavesurfer.currentPlayer && AudioWavesurfer.currentPlayer !== this.wavesurfer) {
            AudioWavesurfer.currentPlayer.pause();
          }
          if (AudioWavesurfer.currentPlayButtonIcon && AudioWavesurfer.currentPlayButtonIcon !== this.playButtonIcon) {
            AudioWavesurfer.currentPlayButtonIcon.classList.remove('icon-pause2');
            AudioWavesurfer.currentPlayButtonIcon.classList.add('icon-play3');
          }
          this.wavesurfer.play();
          AudioWavesurfer.currentPlayer = this.wavesurfer;
          AudioWavesurfer.currentPlayButtonIcon = this.playButtonIcon;
          this.playButtonIcon.classList.remove('icon-play3');
          this.playButtonIcon.classList.add('icon-pause2');
        }
      }

      formatTimecode(seconds) {
        const minutes = Math.floor(seconds / 60);
        const secs = Math.floor(seconds % 60);
        return `${minutes}:${secs < 10 ? '0' + secs : secs}`;
      }

      setupWavesurferEventListeners() {
        this.wavesurfer.on("ready", () => {
          const duration = this.wavesurfer.getDuration();
          this.totalDuration.innerHTML = this.formatTimecode(duration);
        });

        this.wavesurfer.on("audioprocess", () => {
          const time = this.wavesurfer.getCurrentTime();
          this.currentTime.innerHTML = this.formatTimecode(time);
        });

        this.wavesurfer.on("finish", () => {
          this.playButtonIcon.classList.remove('icon-pause2');
          this.playButtonIcon.classList.add('icon-play3');
        });
      }

      setupEventListeners() {
        this.playButton.addEventListener("click", () => this.togglePlay());

        if(!this.waveformPeaks) {
          this.wavesurfer.load(this.pathAudio);
        }

        this.setupWavesurferEventListeners();
      }
    }

    window.AudioWavesurfer = AudioWavesurfer;

  }).catch((error) => {
    console.error('Error on library loading :', error);
  });

}(drupalSettings));
