(function (drupalSettings) {
  let libraryPath = drupalSettings.audio_wavesurfer.library_path;
  const wavesurferRegionsPath = libraryPath + '/plugins/regions.esm.js';

  import(wavesurferRegionsPath).then((Regions) => {

    /**
     * Audio wavesurfer region class.
     */
    class AudioWaveSurferRegion {
      constructor(wavesurfer, containerElement) {
        this.wavesurfer = wavesurfer;
        this.containerElement = containerElement;
        this.wsRegions = this.wavesurfer.registerPlugin(Regions.default.create());
        this.sliderContainer = document.querySelector('#tabs-clip-audio');
        this.clipElements = this.sliderContainer.querySelectorAll('.clip-infos');
        this.buttonsContainer = document.querySelector('.play-regions-button-container');
        this.colorsTransparent = [
          "rgba(89, 205, 144, 0.1)", // Emerald
          "rgba(63, 167, 214, 0.1)", // Picton Blue
          "rgba(125, 91, 166, 0.1)", // Royal Purple
          "rgba(252, 100, 113, 0.1)", // Crayota
        ];
        this.colors = [
          "rgba(89, 205, 144, 1)",
          "rgba(63, 167, 214, 1)",
          "rgba(125, 91, 166, 1)",
          "rgba(252, 100, 113, 1)",
        ];
        this.initRegions();
        this.initEventListeners();
      }

      initRegions() {
        this.wavesurfer.on('decode', () => {
          this.clipElements.forEach((rangeInput, index) => {
            const clipName = rangeInput.dataset.clipId;
            let minDurationInput = rangeInput.querySelectorAll('.min-duration-input');
            const minDuration = minDurationInput[0].dataset.clipMinDuration;
            let maxDurationInput = rangeInput.querySelectorAll('.max-duration-input');
            const maxDuration = maxDurationInput[0].dataset.clipMaxDuration;

            const region = this.wsRegions.addRegion({
              start: minDurationInput[0].value,
              end: maxDurationInput[0].value,
              minLength: minDuration,
              maxLength: maxDuration ?? this.wavesurfer.getDuration(),
              content: clipName,
              id: 'region_' + clipName,
              color: this.colorsTransparent[index % this.colorsTransparent.length],
            });

            region.on('update', () => {
              minDurationInput[0].value = region.start;
              maxDurationInput[0].value = region.end;
            });

            this.createPlayButton(clipName, region, this.colors[index % this.colors.length]);
          });
        });
      }

      createPlayButton(clipName, region, color) {
        const playButton = document.createElement('button');
        const icon = document.createElement("i");
        playButton.textContent = clipName;
        playButton.classList.add('play-button');
        playButton.setAttribute('id', clipName);
        playButton.setAttribute('type', 'button')
        playButton.style.backgroundColor = color;
        icon.className = "icon-play3";
        playButton.prepend(icon);

        playButton.addEventListener('click', () => {
          region.play();
          this.containerElement.setAttribute("played-region", region.end);
        });

        this.buttonsContainer.appendChild(playButton);
      }

      initEventListeners() {
        this.wavesurfer.on("audioprocess", () => {
          const time = this.wavesurfer.getCurrentTime();
          const currentRegion = this.containerElement.getAttribute("played-region");
          if (currentRegion && time > currentRegion) {
            this.wavesurfer.pause();
          }
        });
      }

    }

    window.AudioWaveSurferRegion = AudioWaveSurferRegion;
  });

}(drupalSettings));
