<?php

namespace Drupal\audio_wavesurfer_clips\Element;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\Attribute\FormElement;
use Drupal\Core\Render\Element\FormElementBase;

/**
 * Provides a form element for Audio Wavesurfer Clip.
 */
#[FormElement('audio_wavesurfer_clip')]
class AudioWavesurferClip extends FormElementBase {

  /**
   * {@inheritdoc}
   */
  public function getInfo() {
    $class = static::class;
    return [
      '#tree' => TRUE,
      '#audio_clip_type' => [],
      '#audio_clip_storage' => [],
      '#file_upload_id' => NULL,
      '#file_upload_url' => NULL,
      '#file_upload_duration' => NULL,
      '#process' => [
        [$class, 'processAudioClipActions'],
      ],
    ];
  }

  /**
   * Processes a form AudioClipActions element.
   */
  public static function processAudioClipActions(array &$element, FormStateInterface $form_state, array &$complete_form) {
    $audio_clips_storage = $element['#audio_clip_storage'];
    $audio_clips_type = $element['#audio_clip_type'];
    $file_upload_id = $element['#file_upload_id'];
    $file_upload_url = $element['#file_upload_url'];
    $file_upload_duration = $element['#file_upload_duration'];

    $element['audio_player'] = [
      '#theme' => 'audio_file__wavesurfer',
      '#clips' => array_keys($audio_clips_type),
      '#file_upload_id' => $file_upload_id,
      '#file_upload_url' => $file_upload_url,
      '#attached' => [
        'library' => ['audio_wavesurfer_clips/audio_clip'],
      ],
    ];

    $element['clips'] = [
      '#type' => 'horizontal_tabs',
      '#title' => 'Clips',
      '#tree' => TRUE,
    ];

    $element['clips']['#attributes'] = [
      'id' => 'tabs-clip-audio',
      'class' => 'disable',
    ];

    foreach ($audio_clips_type as $audio_clip_type => $audio_clip_attributes) {
      $start_time_storage = NULL;
      $end_time_storage = NULL;
      if (array_key_exists($audio_clip_type, $audio_clips_storage) && $audio_clips_storage[$audio_clip_type]) {
        $start_time_storage = $audio_clips_storage[$audio_clip_type]->get('start_time')->value;
        $end_time_storage = $audio_clips_storage[$audio_clip_type]->get('end_time')->value;
      }

      $start_time_value = $start_time_storage ?? 0;
      $end_time_value = $end_time_storage;
      $min_duration = empty($audio_clip_attributes['min_duration']) ? 1 : $audio_clip_attributes['min_duration'];
      $max_duration = empty($audio_clip_attributes['max_duration']) ? intval($file_upload_duration / 2) : $audio_clip_attributes['max_duration'];

      if (!$end_time_value) {
        $end_time_value = $max_duration;
      }

      $element['clips'][$audio_clip_type] = [
        '#type' => 'details',
        '#title' => $audio_clip_attributes['label'],
        '#group' => 'tabs',
        '#attributes' => [
          'class' => ['clip-infos'],
          'data-clip-id' => $audio_clip_type,
        ],
      ];

      $element['clips'][$audio_clip_type]['start_time'] = [
        '#type' => 'range',
        '#min' => 0,
        '#max' => $file_upload_duration,
        '#step' => 1,
        '#default_value' => $start_time_value,
        '#attributes' => [
          'class' => ['min-duration-input'],
          'data-clip-min-duration' => $min_duration,
        ],
      ];

      $element['clips'][$audio_clip_type]['end_time'] = [
        '#type' => 'range',
        '#min' => 0,
        '#max' => $file_upload_duration,
        '#step' => 1,
        '#default_value' => $end_time_value,
        '#attributes' => [
          'class' => ['max-duration-input'],
          'data-clip-max-duration' => $max_duration,
        ],
      ];
    }

    return $element;
  }

}
