<?php

namespace Drupal\audio_wavesurfer\Entity;

use Drupal\Core\Entity\ContentEntityBase;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\file\Entity\File;

/**
 * Defines the waveform entity class.
 *
 * @ContentEntityType(
 *   id = "waveform",
 *   label = @Translation("waveform"),
 *   label_collection = @Translation("waveform"),
 *   label_singular = @Translation("waveform item"),
 *   label_plural = @Translation("waveform items"),
 *   label_count = @PluralTranslation(
 *     singular = "@count waveform item",
 *     plural = "@count waveform items"
 *   ),
 *   base_table = "waveform",
 *   entity_keys = {
 *     "id" = "id",
 *   }
 * )
 */
class Waveform extends ContentEntityBase implements ContentEntityInterface {

  /**
   * {@inheritdoc}
   */
  public static function preDelete(EntityStorageInterface $storage, array $entities) {
    parent::preDelete($storage, $entities);

    foreach ($entities as $entity) {
      $waveform_fid = $entity->get('waveform_fid')->value;
      $waveform_file = File::load($waveform_fid);
      $waveform_file?->delete();
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function baseFieldDefinitions(EntityTypeInterface $entity_type) {
    $fields['id'] = BaseFieldDefinition::create('integer')
      ->setLabel(t('ID'))
      ->setDescription(t('The ID of the waveform entity.'))
      ->setReadOnly(TRUE);

    $fields['audio_fid'] = BaseFieldDefinition::create('integer')
      ->setLabel(t('Audio FID'))
      ->setDescription(t('The audio file ID.'))
      ->setRequired(TRUE);

    $fields['waveform_fid'] = BaseFieldDefinition::create('integer')
      ->setLabel(t('Waveform FID'))
      ->setDescription(t('The waveform file ID.'))
      ->setRequired(TRUE);

    return $fields;
  }

}
