<?php

namespace Drupal\audio_wavesurfer\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\Attribute\FieldFormatter;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\Plugin\Field\FieldFormatter\EntityReferenceFormatterBase;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\file\Entity\File;

/**
 * Plugin 'audio_wavesurfer' formatter.
 */
#[FieldFormatter(
  id: 'audio_wavesurfer_formatter',
  label: new TranslatableMarkup('Audio Wavesurfer Formatter'),
  description: new TranslatableMarkup('Display the audio file with Wavesurfer library.'),
  field_types: [
    'file',
  ],
)]
class AudioWavesurferFormatter extends EntityReferenceFormatterBase {

  /**
   * Apply this formatter only for entity reference media audio.
   */
  public static function isApplicable(FieldDefinitionInterface $field_definition) {
    $entity_type = $field_definition->get('entity_type');
    $bundle = $field_definition->get('bundle');
    if ($entity_type === 'media' && $bundle === 'audio') {
      return TRUE;
    }

    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $element = [];

    foreach ($items as $delta => $item) {
      $fid = $item->target_id;
      $audio_fid = $fid;

      $audio_file = File::load($audio_fid);
      $audio_file_path = \Drupal::service('file_url_generator')->generateString($audio_file->getFileUri());

      // @todo add audio file name ?
      $element[$delta] = [
        '#theme' => 'audio_file__wavesurfer',
        '#file_upload_url' => $audio_file_path,
        '#file_upload_id' => $audio_fid,
        '#attached' => [
          'library' => ['audio_wavesurfer/audio'],
        ],
      ];

      // Manage waveform JSON.
      $wavesurfer_config = \Drupal::config('audio_wavesurfer.settings');
      $waveform_options = $wavesurfer_config->get('waveform_options');
      if (array_key_exists('waveform_usage', $waveform_options) && $waveform_options['waveform_usage']) {
        $waveform_peaks = \Drupal::service('audio_wavesurfer.waveform_json_service')->getAudioWaveformStored($audio_fid);
        if ($waveform_peaks) {
          $element[$delta]['#attached']['drupalSettings'] = [
            'audio_wavesurfer' => [
              'waveform_peaks' => $waveform_peaks,
            ],
          ];
        }
      }

      // Add config wavesurfer options.
      $element[$delta]['#attached']['drupalSettings']['audio_wavesurfer']['options'] = [
        'color_options' => $wavesurfer_config->get('color_options'),
        'bar_options' => $wavesurfer_config->get('bar_options'),
      ];
    }

    return $element;
  }

}
